unit ufrmJsonTest;

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Controls, FMX.Forms, FMX.Graphics, FMX.Dialogs, FMX.Memo.Types,
  FMX.StdCtrls, FMX.Controls.Presentation, FMX.ScrollBox, FMX.Memo,
  Aurelius.Mapping.Attributes,
  Aurelius.Types.Blob,
  Aurelius.Types.DynamicProperties,
  Aurelius.Types.Nullable,
  Aurelius.Types.Proxy,
  System.JSON,
  Aurelius.Json.DataSnap;

type
  [Entity]
  [Table('Product')]
  [Description('Produto')]
  [Sequence('Id_Product')]
  [Id('FId', TIdGenerator.IdentityOrSequence)]
  TProduct = class
  private
    [Column('Id', [TColumnProp.Required])]
    [Description('')]
    FId: Integer;

    [Column('Description', [TColumnProp.Required], 30)]
    [Description('')]
    FDescription: string;

    [Column('Price', [TColumnProp.Required])]
    [Description('')]
    FPrice: Double;
  public
    property Id: Integer read FId write FId;
    property Description: string read FDescription write FDescription;
    property Price: Double read FPrice write FPrice;

    function ToString: string;
  end;


type
  TfrmJsonTest = class(TForm)
    mmLog: TMemo;
    ToJson: TButton;
    FromJson: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ToJsonClick(Sender: TObject);
    procedure FromJsonClick(Sender: TObject);
  private
    { Private declarations }
    FSerializer: TDataSnapJsonSerializer;
    FDeserializer: TDataSnapJsonDeserializer;

    FProduct : TProduct;

    FJsonValue : TJsonValue;
  public
    { Public declarations }
  end;

var
  frmJsonTest: TfrmJsonTest;

implementation

{$R *.fmx}

procedure TfrmJsonTest.FormCreate(Sender: TObject);
begin
  FSerializer := TDataSnapJsonSerializer.Create;
  FDeserializer := TDataSnapJsonDeserializer.Create;

  FProduct := TProduct.Create;
  FProduct.Id := 1;
  FProduct.Description := 'Bread';
  FProduct.Price := 0.25;

  mmLog.Text := FProduct.ToString + sLineBreak + sLineBreak;
end;

procedure TfrmJsonTest.FormDestroy(Sender: TObject);
begin
  FSerializer.Free;
  FDeserializer.Free;
end;

procedure TfrmJsonTest.FromJsonClick(Sender: TObject);
var
  ftmpproduct : TProduct;
begin
  mmLog.Text := mmLog.Text + 'Deserialize TProduct' + sLineBreak;
  try
    ftmpproduct := FDeserializer.FromJson<TProduct>(FJsonValue);
    mmLog.Text := mmLog.Text + ftmpproduct.ToString + sLineBreak;
  except
     on E : Exception do
    begin
      mmLog.Text := mmLog.Text + E.Message + sLineBreak;
    end;
  end;
end;

procedure TfrmJsonTest.ToJsonClick(Sender: TObject);
begin
  mmLog.Text := mmLog.Text + 'Serialize TProduct' + sLineBreak;
  FJsonValue := FSerializer.ToJson(FProduct);
  mmLog.Text := mmLog.Text + FJsonValue.ToString + sLineBreak + sLineBreak;
end;



{ TProduct }

function TProduct.ToString: string;
begin
  result := Format('TProduct: { Id (integer): %d; Description (string): %s; Price (double): %.2f; }',
    [FId, FDescription, FPrice]);
end;

end.
